'''Author: Nick Hillier'''
# Various utils for interacting with ROS

import roslib
roslib.load_manifest('mrol_mapping')
import rospy

import numpy as np
import tf
import mrol_mapping.poseutil as poseutil

def makePose3D(transform):
    p = poseutil.Pose3D()
    R = tf.transformations.quaternion_matrix(transform[1])
    T = transform[0]
    #M = [R,T;0,0,0,0]
    M = np.zeros([4,4])
    M = R
    M[0:3,3:4] = np.asarray(T).reshape(3,1)    
    p.setMatrix(M)
    return p

from sensor_msgs.msg import PointCloud2
from sensor_msgs.msg import PointField 
def xyzs2pc2(xyzs,stamp,seq,frame_id):
    #import pydb;pydb.set_trace()
    pc2Data = PointCloud2()
    if len(xyzs) != 0:
        if type(xyzs[0][0]) == np.float64:
            xyzs = np.array(xyzs,dtype=np.float32)
        assert (xyzs.shape[1] != 3) or (xyzs.shape[1] != 6)
        if xyzs.shape[1] == 3:
            numfields = 3
        else:
            numfields = 4
        float32LE = np.dtype(np.float32,align='<')
        pc2Data.is_dense = False
        pc2Data.is_bigendian = False
        pc2Data.height = 1
        pc2Data.width = len(xyzs)
        pc2Data.header.frame_id = frame_id
        pc2Data.header.seq = seq
        pc2Data.header.stamp = stamp;
        for i in range(numfields):
            pc2Field = PointField()
            if i == 0:
                pc2Field.name = 'x'
            elif i == 1:
                pc2Field.name = 'y'
            elif i == 2:
                pc2Field.name = 'z'
            elif i == 3:
                pc2Field.name = 'rgb'
            else:
                pc2Field.name = 'unknown'
            pc2Field.count = 1
            #if type(xyzs[0][0]) == np.float64:
            #    pc2Field.datatype = 8 # FLOAT64
            #    dataLen = 8 # FLOAT64
            if type(xyzs[0][0]) == float32LE:
                pc2Field.datatype = 7 # FLOAT32
                dataLen = 4 # FLOAT32
            else:
                assert False, "unknown datatype"
            
            pc2Field.offset = i*dataLen
            pc2Data.fields.append(pc2Field)
        pc2Data.point_step = i*dataLen + dataLen;
        pc2Data.row_step = pc2Data.point_step * pc2Data.width;
        if xyzs.shape[1] == 3:
            data_buf = xyzs.flatten().tostring()
        else:
            #import pydb;pydb.set_trace()
            rgb = np.hstack([np.uint8(xyzs[:,6:2:-1]),np.zeros([xyzs.shape[0],1],dtype=np.uint8)]).copy()
            rgb = rgb.view(dtype=float32LE)
            data = np.hstack([xyzs[:,0:3],rgb])
            data_buf = data.flatten().tostring()
        if (xyzs.shape[1] == 3):
            assert len(data_buf)/float(dataLen)/3.0 == len(xyzs), "len(data_buf)/4.0/3.0 = "+np.str(len(data_buf)/4.0/3.0)+" but len(xyzs) = "+np.str(len(xyzs))
        else:
            assert len(data_buf)/float(dataLen)/4.0 == len(xyzs), "len(data_buf)/4.0/4.0 = "+np.str(len(data_buf)/4.0/4.0)+" but len(xyzs) = "+np.str(len(xyzs))
        pc2Data.data = data_buf
    return pc2Data
    
def pc22xyzs(msg, max_range=100.):
    #import pydb;pydb.set_trace()
    assert msg.is_bigendian == False, "Big Endian support is not yet implemented"
    num_fields = len(msg.fields)
    assert num_fields >=3, "Message must contain at least 3 dimensions (x,y,z)"
    # TODO: checks on fields content for type etc
    float32LE = np.dtype(np.float32,align='<')
    #data = np.frombuffer(msg.data, dtype=float32LE)
    #data_arr = data.reshape((-1,num_fields*2))
    data = np.frombuffer(msg.data, dtype=np.uint8)
    data_arr = data.reshape((-1,msg.point_step))
    # TODO: WARNING! The copying below assumes msg.fields is x,y,z(,rgb).
    #xyzsD = data_arr[:,0:3].copy()
    data_arrF = data_arr.view(dtype=float32LE)
    xyzsD = data_arrF[:,0:3]
    inds = np.logical_not(np.isnan(xyzsD[:,1]))
    xyzsD = xyzsD[inds,:]
    ranges = (xyzsD[:,0]*xyzsD[:,0] + xyzsD[:,1]*xyzsD[:,1]) + xyzsD[:,2]*xyzsD[:,2]
    indsD = np.logical_and(ranges < max_range**2, ranges > 0)
    xyzsD = xyzsD[indsD,:]
    if num_fields == 4:
        # TODO: This indexing ONLY WORKS FOR KINECT!!!
        #import pydb;pydb.set_trace()
        #xyzsRGB = data_arrF[:,4] 
        xyzsRGB = data_arr[:,18:15:-1] # RGB is backwards!
        xyzsRGB = xyzsRGB[inds,:]
        xyzsRGB = xyzsRGB[indsD,:]
        #xyzsRGB = xyzsRGB.view(np.uint8)
        #xyzsRGB = xyzsRGB.reshape((-1,4))
        
        xyzs = np.hstack([xyzsD,xyzsRGB])
    else:
        xyzs = xyzsD
    
    return xyzs
