// This file is a modified version of that contained in the openrave 
// software, see below license details

// Copyright (c) 2010, CSIRO Autonomous Systems Lab, Nick Hillier

// Copyright (C) 2006-2009 Rosen Diankov (rdiankov@cs.cmu.edu)
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
#include <odephysics_ORplugin/csiroplugindefs.h>

#include <odephysics_ORplugin/csiro_odephysics.h>
#include <odephysics_ORplugin/csiro_odecontroller.h>
#include <rave/plugin.h>

static list< boost::shared_ptr<void> >* s_listRegisteredReaders=NULL;
InterfaceBasePtr CreateInterfaceValidated(InterfaceType type, const std::string& interfacename, std::istream& sinput, EnvironmentBasePtr penv)
{
    if( !s_listRegisteredReaders) {
        s_listRegisteredReaders = new list< boost::shared_ptr<void> >();
        s_listRegisteredReaders->push_back(RaveRegisterXMLReader(OpenRAVE::PT_PhysicsEngine,"csiro_odeproperties",CustomODEPhysics::CreateXMLReader));
    }

    switch(type) {
    case OpenRAVE::PT_PhysicsEngine:
        if( interfacename == "csiro_ode" )
            return InterfaceBasePtr(new CustomODEPhysics(penv));
        break;
    case OpenRAVE::PT_Controller:
        //printf("Interface name to match = %s\n",interfacename.c_str());
        if(  (interfacename == "csiro_odevelocity")){
            return InterfaceBasePtr(new ODEVelocityController(penv));
        }    
        break;
    default:
        break;
    }

    return InterfaceBasePtr();
}

void GetPluginAttributesValidated(PLUGININFO& info)
{
	info.interfacenames[OpenRAVE::PT_PhysicsEngine].push_back("csiro_ode");
    info.interfacenames[OpenRAVE::PT_Controller].push_back("csiro_odevelocity");
}

RAVE_PLUGIN_API void DestroyPlugin()
{
    delete s_listRegisteredReaders;
    s_listRegisteredReaders = NULL;
}
